import os
import json
from pkg_resources import resource_filename

from docutils import nodes
from docutils.utils import new_document

from sphinx.transforms import SphinxTransform
from sphinx.util.docutils import LoggingReporter
from sphinx.util.fileutil import copy_asset

from . import __version__


class EmojiSubstitutions(SphinxTransform):
    default_priority = 211

    def __init__(self, document, startnode=None):
        super().__init__(document, startnode)
        self.parser = self.app.registry.create_source_parser(self.app, 'rst')

    def apply(self):
        config = self.document.settings.env.config
        settings, source = self.document.settings, self.document['source']
        codes = resource_filename(__name__, 'codes.json')

        with open(codes, encoding='utf-8') as fp:
            replacements = json.load(fp)

        to_handle = (set(replacements.keys()) -
                     set(self.document.substitution_defs))

        for ref in self.document.traverse(nodes.substitution_reference):
            refname = ref['refname']
            if refname in to_handle:
                text = replacements[refname]

                doc = new_document(source, settings)
                doc.reporter = LoggingReporter.from_reporter(doc.reporter)
                self.parser.parse(text, doc)

                substitution = doc.next_node()
                # Remove encapsulating paragraph
                if isinstance(substitution, nodes.paragraph):
                    substitution = substitution.next_node()
                ref.replace_self(substitution)


def copy_asset_files(app, exc):
    asset_files = [
        resource_filename(__name__, 'twemoji.js'),
        resource_filename(__name__, 'twemoji.css'),
    ]
    if exc is None:  # build succeeded
        for path in asset_files:
            copy_asset(path, os.path.join(app.outdir, '_static'))


def setup(app):
    app.connect('build-finished', copy_asset_files)
    style = app.config._raw_config.get('sphinxemoji_style')
    if style == 'twemoji':
        app.add_js_file('https://twemoji.maxcdn.com/v/latest/twemoji.min.js')
        app.add_js_file('twemoji.js')
        app.add_css_file('twemoji.css')
    app.add_transform(EmojiSubstitutions)
    return {'version': __version__, 'parallel_read_safe': True}



import json
from urllib.request import urlopen


def get_gemojione():
    remote = 'https://raw.githubusercontent.com/bonusly/gemojione/master'
    data = json.load(urlopen(remote + '/config/index.json'))
    codes = {}
    for emoji in data.values():
        codes[emoji['shortname']] = emoji['moji']
        for alias in emoji['aliases']:
            codes[alias] = emoji['moji']
    return codes


def get_joypixels():
    remote = 'https://raw.githubusercontent.com/joypixels/emoji-toolkit/master'
    data = json.load(urlopen(remote + '/emoji.json'))
    codes = {}
    for info in data.values():
        hexstring = info['code_points']['fully_qualified']
        emoji = ''.join(chr(int(c, 16)) for c in hexstring.split('-'))
        codes[info['shortname']] = emoji
        for alias in info['shortname_alternates']:
            codes[alias] = emoji
    return codes


def update_codes():
    with open('sphinxemoji/codes.json', encoding='utf-8') as current:
        codes = json.load(current)
    for getter in [
        get_gemojione,
        get_joypixels,
    ]:
        codes.update(getter())
    with open('sphinxemoji/codes.json', 'w', encoding='utf-8') as output:
        json.dump(codes, output, sort_keys=True, indent=4, ensure_ascii=False)
        output.write('\n')


if __name__ == '__main__':
    update_codes()
